'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _net = require('net');

var _net2 = _interopRequireDefault(_net);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _nodeZookeeperClient = require('node-zookeeper-client');

var _nodeZookeeperClient2 = _interopRequireDefault(_nodeZookeeperClient);

var _ip = require('ip');

var _ip2 = _interopRequireDefault(_ip);

var _service = require('./service');

var _service2 = _interopRequireDefault(_service);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class Dubbo {
  constructor({
    application,
    services = {},
    root = 'dubbo',
    version = '2.5.3.6',
    zookeeper,
    pool
  }) {

    this._root = root;
    this._version = version;
    this._connected = false;
    this._pendingServices = [];
    this._services = {};

    this._ip = _ip2.default.address();
    this._baseQuery = {
      application,
      category: 'consumers',
      check: 'false',
      dubbo: version,
      side: 'consumer'
    };

    this._pool = pool;

    const zk = this._zookeeper = _nodeZookeeperClient2.default.createClient(zookeeper.host, zookeeper);

    zk.connect();
    zk.once('connected', () => {
      this._pendingServices.forEach(name => {
        this._services[name].setup();
      });
    });

    Object.keys(services).forEach(name => {
      const options = services[name];
      this.register(name, options);
    });
  }

  // Get a service
  service(name) {
    const existing = this._services[name];
    if (existing) {
      return existing;
    }

    const service = this._services[name] = new _service2.default(name);

    if (!this._connected) {
      this._pendingServices.push(name);
    } else {
      service.setup();
    }

    return service;
  }

  // Register a service
  register(name, options) {
    this.service(name).register({
      options,
      path: {
        consumer: this._consumerPath(options),
        provider: this._providerPath(options)
      },
      zookeeper: this._zookeeper,
      dubboVersion: this._version,
      pool: this._pool
    });
    return this;
  }

  _consumerPath({
    interface: _interface,
    version,
    group
  }) {

    const path = _url2.default.format({
      protocol: 'consumer',
      slashes: 'true',
      host: `${this._ip}/${_interface}`,
      query: _extends({}, this._baseQuery, {
        interface: _interface,
        revision: version,
        version,
        group,
        timestamp: Date.now()
      })
    });

    return `/${this._root}/${_interface}/consumers/${encodeURIComponent(path)}`;
  }

  _providerPath({
    interface: _interface
  }) {

    return `/${this._root}/${_interface}/providers`;
  }
}
exports.default = Dubbo;
module.exports = exports['default'];