'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _events = require('events');

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _debug2 = require('debug');

var _debug3 = _interopRequireDefault(_debug2);

var _nodeZookeeperClient = require('node-zookeeper-client');

var _utils = require('./utils');

var _client = require('./client');

var _encoder = require('./encoder');

var _encoder2 = _interopRequireDefault(_encoder);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug3.default)('dubbo');

class Service extends _events.EventEmitter {
  constructor(name) {
    super();

    this._name = name;
    this._hosts = [];
    this._methods = {};
    this._setupError = null;
    this._ready = false;
  }

  // Register service informations
  register({
    zookeeper,
    options: {
      interface: _interface,
      version,
      group,
      timeout = 6000
    },
    dubboVersion,
    path: {
      consumer,
      provider
    },
    pool
  }) {

    if (this._registered) {
      return;
    }

    this._registered = true;

    this._zookeeper = zookeeper;
    this._version = version;
    this._group = group;
    this._consumer = consumer;
    this._provider = provider;
    this._err = null;
    this._encoder = new _encoder2.default({
      dubboVersion,
      interface: _interface,
      version,
      group,
      timeout
    });

    this._client = pool ? new _client.ClientWithPool(pool) : new _client.Client();
  }

  // Setup service
  setup() {
    return Promise.all([this._createNode(), this._initProvider()]).then(() => {
      this._ready = true;
      this.emit('ready');
    }).catch(err => {
      this._setupError = err;
      this._ready = true;
      this.emit('ready');
    });
  }

  // @returns `Promise`
  _createNode() {
    const zk = this._zookeeper;
    const consumer = this._consumer;

    return new Promise((resolve, reject) => {
      zk.exists(consumer, (err, stat) => {
        if (err) {
          return reject(err);
        }

        if (stat) {
          return resolve();
        }

        zk.create(consumer, _nodeZookeeperClient.CreateMode.EPHEMERAL, (err, node) => {
          // Skip node exists error for concurrency
          if (err && err.getCode() === _nodeZookeeperClient.Exception.NODE_EXISTS) {
            return resolve();
          }

          if (err) {
            return reject(err);
          }

          resolve();
        });
      });
    });
  }

  _initProvider() {
    return new Promise((resolve, reject) => {
      this._zookeeper.getChildren(this._provider, () => this._initProvider(), (err, children) => {
        if (err) {
          err.code = 'ERR_GET_CHILDREN';
          return reject(err);
        }

        resolve(children);
      });
    }).then(children => this._applyProviders(children));
  }

  _applyProviders(children) {
    if (!children) {
      return (0, _utils.reject)(`no children found for "${this._provider}"`, 'NO_CHILDREN');
    }

    const available = children.map(child => _querystring2.default.parse(decodeURIComponent(child))).filter(zoo => zoo.version === this._version && zoo.group === this._group);

    if (!available.length) {
      return (0, _utils.reject)(`no available children found for "${this._provider}"`, 'NO_AVAILABLE_CHILDREN');
    }

    this._hosts = available.map(zoo => _url2.default.parse(Object.keys(zoo)[0]).host);

    const methodsHash = {};
    available.forEach(zoo => {
      zoo.methods.split(',').forEach(method => {
        methodsHash[method] = true;
      });
    });

    this._methods = methodsHash;
  }

  _selectHost() {
    return this._hosts[Math.random() * this._hosts.length | 0].split(':');
  }

  invoke(method, ...args) {
    if (!this._registered) {
      return (0, _utils.reject)(`service "this._name" must be registered before use`, 'SERVICE_NOT_REGISTERED');
    }

    if (this._ready) {
      return this._invoke(method, args);
    }

    return new Promise((resolve, reject) => {
      this.once('ready', () => {
        this._invoke(method, args).then(resolve, reject);
      });
    });
  }

  _invoke(method, args) {
    if (this._setupError) {
      return Promise.reject(this._setupError);
    }

    if (!this._methods[method]) {
      return (0, _utils.reject)(`"${method}" is not an available method`);
    }

    const buffer = this._encoder.encode(method, args);

    var _selectHost = this._selectHost(),
        _selectHost2 = _slicedToArray(_selectHost, 2);

    const host = _selectHost2[0],
          port = _selectHost2[1];


    return this._client.request(host, port, buffer).catch(err => {
      if (err.code !== 'SOCKET_ERROR') {
        return Promise.reject(err);
      }

      return this._initProvider().then(() => {
        return this._invoke(method, args);
      });
    });
  }
}
exports.default = Service;
module.exports = exports['default'];